@@ -21,6 +21,7 @@ class WorkspaceRun:
         parser.add_argument('--workspaceRunId', default=None)
         parser.add_argument('--codePath', default=None)
         parser.add_argument('--stopFlag', default=False)
+        parser.add_argument('--stopTimeout', default=60)
 
         args, unknown = parser.parse_known_args()
         args_dict = vars(args)
@@ -28,6 +29,7 @@ class WorkspaceRun:
         self.code_path = args_dict['codePath']
         self.workspace_run_log.workspace_run_id = args_dict['workspaceRunId']
         self.workspace_run_log.stop_flag = args_dict['stopFlag']
+        self.workspace_run_log.stop_timeout = args_dict['stopTimeout']
 
         if not self.code_path:
             raise Exception("codePath cannot be none")
