@@ -1,6 +1,6 @@
 ENV_PROJECT_ID='MONITORING_DEPLOY_PROJECT_ID'
 ENV_DEPLOY_ID='MONITORING_DEPLOY_DEPLOY_ID'
 
-BACK_END_API_URL = 'api.accu-rpm'
+BACK_END_API_URL = 'api.accu'
 BACK_END_API_PORT = '8080'
 BACK_END_API_URI = ''
\ No newline at end of file
