@@ -53,7 +53,7 @@ def shap_value(model, dat, feature_name):
         for i in range(shap_values.shape[1]):
             feature_contribution[feature_name[i]] = np.mean(np.abs(shap_values[:,i])) 
     else:
-        raise ValueError('The dimensions of the train data and the feature name do not match.')
+        raise ValueError("The dimensions of the train data and the feature name do not match. \nPlease use 'set_feature()' function for setting feature names.")
         
     return(dict(feature_contribution))
 
